CREATE OR REPLACE VIEW {{ view_name }} AS (
    SELECT * FROM (
        VALUES
        {%- for row in dataset %}
        (
            {%- for field in row -%}
            '{{ field }}'
            {%- if not loop.last -%}
            , 
            {%- endif -%}
            {%- endfor -%}
        )
        {%- if not loop.last -%}
        , 
        {%- endif -%}
        {%- endfor %}
    )
    AS t -- noqa: L025
    (
        {%- for col in view_cols -%}
        {{ col }}
        {%- if not loop.last -%}
        , 
        {%- endif -%}
        {%- endfor -%}
    )
);
