"""Error types"""


class LibraryError(Exception):
    """
    Package level error
    """

    pass


class LibrarySchemaError(Exception):
    """
    Package level error
    """


class StudyManifestParsingError(Exception):
    """Basic error for StudyManifestParser"""
